//------------------------------------------------------------------------
//
// OpenDCC - OpenDecoder2
//
// Copyright (c) 2006 Kufer
//
// This source file is subject of the GNU general public license 2,
// that is available at the world-wide-web at
// http://www.gnu.org/licenses/gpl.txt
// 
//------------------------------------------------------------------------
//
// file:      hardware.h
// author:    Wolfgang Kufer
// contact:   kufer@gmx.de
// webpage:   http://www.opendcc.de
// history:   2006-02-14 V0.1 kw copied from opendecoder.c
//            2007-05-21 V0.2 kw added OpenDecoder3
//
//------------------------------------------------------------------------
//
// purpose:   flexible general purpose decoder for dcc
//            here: hardware definitions
//
//
//------------------------------------------------------------------------
#ifndef _HARDWARE_H_
#define _HARDWARE_H_

#ifndef TARGET_HARDWARE
	#warning: SEVERE: no target hardware defined! Code will not work!
#endif


//========================================================================

#if (TARGET_HARDWARE == OPENDECODER2) 

//---------------------------------------------------------------------
// CPU Definitions:
//
#define  PROCESSOR	ATMEGA8       
// if changed: check timings, timer settings, baudrate

#if (PROCESSOR == ATMEGA8)
  // atmega8:		   1024 Byte SRAM, 512 Byte EEPROM
  #define SRAM_SIZE    1024
  #define EEPROM_SIZE  512
  #define EEPROM_BASE  0x810000L

#elif (PROCESSOR == ATMEGA8515)
  // atmega8515:   512 Byte SRAM, 512 Byte EEPROM
  #define SRAM_SIZE    512
  #define EEPROM_SIZE  512
  #define EEPROM_BASE  0x810000L
#endif

#ifndef F_CPU
	// prevent compiler error by supplying a default 
	# warning "F_CPU not defined", set default to 8MHz 
	# define F_CPU 8000000UL
	// if changed: check every place where it is used
	// (possible range underflow/overflow in preprocessor!)
#endif

//---------------------------------------------------------------------------
// PORT Definitions:
//
// PORTB:
#define SHIFT_DDR				DDRB
#define SHIFT_PORT			PORTB
#define SHIFT_SCK				PB5		// Schiebetakt 
#define SHIFT_MISO			PB4		// Serieller Eingang, unused
#define SHIFT_MOSI			PB3		// Serieller Ausgang
#define SHIFT_RCK				PB2		// Speichertakt ~SS
#define SHIFT_SCLR			PB1		// Reset fr Schieberegister  ~SCLR
#define SHIFT_OE				PB0		// Ausgangssteuerung 'OUTPUT ENABLE' ~OE

#define SHIFT_RCK_ON		SHIFT_PORT |=	 (1<<SHIFT_RCK)
#define SHIFT_RCK_OFF		SHIFT_PORT &= ~(1<<SHIFT_RCK)

#define SHIFT_SCLR_OFF	SHIFT_PORT |=	 (1<<SHIFT_SCLR)		// invertiert !
#define SHIFT_SCLR_ON		SHIFT_PORT &= ~(1<<SHIFT_SCLR)

#define SHIFT_OE_OFF		SHIFT_PORT |=	 (1<<SHIFT_OE)			// invertiert !
#define SHIFT_OE_ON			SHIFT_PORT &= ~(1<<SHIFT_OE)

// PORTC:
// unused - all ports are routed to vias

// PORTD:
#define DCC_DDR					DDRD
#define DCC_PORT				PORTD
#define DCC_PORT_IN			PIND		// must be defined to have portable code
#define DCCIN						PD2			// must be located on INT0
#define JUMPER					PD3			// if Jumper fitted: state is saved in EEPROM
#define PROGTASTER			PD4
#define LED							PD6			// output, 1 turns on LED
#define DCC_ACK					PD7			// output, sending 1 makes an ACK

#define DCCIN_STATE     (DCC_PORT_IN & (1<<DCCIN))
#define JUMPER_FITTED   (!(DCC_PORT_IN & (1<<JUMPER)))
#define PROG_PRESSED    (!(DCC_PORT_IN & (1<<PROGTASTER)))
#define LED_OFF         DCC_PORT &= ~(1<<LED)
#define LED_ON          DCC_PORT |=	 (1<<LED)
#define DCC_ACK_OFF     DCC_PORT &= ~(1<<DCC_ACK)
#define DCC_ACK_ON      DCC_PORT |=	 (1<<DCC_ACK)

// Note LEDs is active high -> state on == pin high!
#define LED_STATE				((DCC_PORT_IN & (1<<LED)))

//----------------------------------------------------------------------------
//
// add Your Target Hardware here!
//
#else
   #warning: SEVERE: unknown target hardware defined! Code will not work!
#endif // TARGET_HARDWARE

#endif // _HARDWARE_
